"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatSnapshot = void 0;
const lsif = __importStar(require("./lsif"));
const Range_1 = require("./Range");
const lsiftyped = lsif.lib.codeintel.lsiftyped;
function formatSnapshot(input, document) {
    const out = [];
    document.occurrences.sort(occurrencesByLine);
    const symbolTable = new Map();
    for (const symbolInfo of document.symbols) {
        symbolTable.set(symbolInfo.symbol, symbolInfo);
    }
    let occurrenceIndex = 0;
    for (const [lineNumber, line] of input.lines.entries()) {
        out.push('  ');
        out.push(line);
        out.push('\n');
        while (occurrenceIndex < document.occurrences.length &&
            document.occurrences[occurrenceIndex].range[0] === lineNumber) {
            const occurrence = document.occurrences[occurrenceIndex];
            occurrenceIndex++;
            if (occurrence.range.length > 3) {
                // Skip multiline occurrences for now.
                continue;
            }
            const range = Range_1.Range.fromLsif(occurrence.range);
            out.push('//');
            out.push(' '.repeat(range.start.character));
            const length = range.end.character - range.start.character;
            if (length < 0) {
                throw new Error(input.format(range, 'negative length occurrence!'));
            }
            out.push('^'.repeat(length));
            out.push(' ');
            const isDefinition = (occurrence.symbol_roles & lsiftyped.SymbolRole.Definition) > 0;
            out.push(isDefinition ? 'definition' : 'reference');
            out.push(' ');
            const symbol = occurrence.symbol.startsWith('scip-typescript npm ')
                ? occurrence.symbol.slice('scip-typescript npm '.length)
                : occurrence.symbol;
            out.push(symbol);
            const info = symbolTable.get(occurrence.symbol);
            if (!isDefinition || !info) {
                out.push('\n');
                continue;
            }
            const prefix = '\n//' + ' '.repeat(range.start.character);
            for (const documentation of info.documentation) {
                out.push(prefix);
                out.push('documentation ');
                out.push(documentation.replace(/\n/g, '\\n'));
            }
            info.relationships.sort((a, b) => a.symbol.localeCompare(b.symbol));
            for (const relationship of info.relationships) {
                out.push(prefix);
                out.push('relationship');
                if (relationship.is_implementation) {
                    out.push(' implementation');
                }
                if (relationship.is_reference) {
                    out.push(' reference');
                }
                if (relationship.is_type_definition) {
                    out.push(' type_definition');
                }
                out.push(' ' + relationship.symbol);
            }
            out.push('\n');
        }
    }
    return out.join('');
}
exports.formatSnapshot = formatSnapshot;
function occurrencesByLine(a, b) {
    return Range_1.Range.fromLsif(a.range).compare(Range_1.Range.fromLsif(b.range));
}
